@@ -19,14 +19,18 @@ package MongoDB::BulkWriteView;
 # ABSTRACT: Bulk write operations against a query document
 
 use version;
-our $VERSION = 'v0.999.998.5';
+our $VERSION = 'v1.0.0';
 
-use Moose;
+use Moo;
 
-use Syntax::Keyword::Junction qw/any/;
 use MongoDB::Error;
-use MongoDB::_Types -types;
-use Types::Standard -types;
+use MongoDB::_Types qw(
+    IxHash
+    Booleanpm
+);
+use Types::Standard qw(
+    InstanceOf
+);
 use boolean;
 use namespace::clean -except => 'meta';
 
@@ -34,7 +38,7 @@ use namespace::clean -except => 'meta';
 has _query => (
     is       => 'ro',
     isa      => IxHash,
-    coerce   => 1,
+    coerce   => IxHash->coercion,
     required => 1
 );
 
@@ -60,8 +64,8 @@ sub upsert {
     return $self->new( %$self, _upsert => true );
 }
 
-sub update {
-    push @_, "update";
+sub update_many {
+    push @_, "update_many";
     goto &_update;
 }
 
@@ -79,7 +83,8 @@ sub _update {
     my $method = pop @_;
     my ( $self, $doc ) = @_;
 
-    unless ( @_ == 2 && ref $doc eq any(qw/HASH ARRAY Tie::IxHash/) ) {
+    my $type = ref $doc;
+    unless ( @_ == 2 && grep { $type eq $_ } qw/HASH ARRAY Tie::IxHash/ ) {
         MongoDB::UsageError->throw("argument to $method must be a single hashref, arrayref or Tie::IxHash");
     }
 
@@ -92,23 +97,12 @@ sub _update {
         $doc = Tie::IxHash->new(%$doc);
     }
 
-    my @keys = $doc->Keys;
-    if ( $method eq 'replace_one' ) {
-        if ( my @bad = grep { substr( $_, 0, 1 ) eq '$' } @keys ) {
-            MongoDB::UsageError->throw("$method document can't have '\$' prefixed field names: @bad");
-        }
-    }
-    else {
-        if ( my @bad = grep { substr( $_, 0, 1 ) ne '$' } @keys ) {
-            MongoDB::UsageError->throw("$method document can't have non- '\$' prefixed field names: @bad");
-        }
-    }
-
     my $update = {
         q      => $self->_query,
         u      => $doc,
-        multi  => $method eq 'update' ? true : false,
+        multi  => $method eq 'update_many' ? true : false,
         upsert => boolean( $self->_upsert ),
+        is_replace => $method eq 'replace_one',
     };
 
     $self->_enqueue_write( [ update => $update ] );
@@ -116,19 +110,29 @@ sub _update {
     return;
 }
 
-sub remove {
+sub delete_many {
     my ($self) = @_;
     $self->_enqueue_write( [ delete => { q => $self->_query, limit => 0 } ] );
     return;
 }
 
-sub remove_one {
+sub delete_one {
     my ($self) = @_;
     $self->_enqueue_write( [ delete => { q => $self->_query, limit => 1 } ] );
     return;
 }
 
-__PACKAGE__->meta->make_immutable;
+#--------------------------------------------------------------------------#
+# Deprecated methods
+#--------------------------------------------------------------------------#
+
+BEGIN {
+    no warnings 'once';
+    *update = \&update_many;
+    *remove = \&delete_many;
+    *remove_one = \&delete_one;
+}
+
 
 1;
 
@@ -144,7 +148,7 @@ MongoDB::BulkWriteView - Bulk write operations against a query document
 
 =head1 VERSION
 
-version v0.999.998.5
+version v1.0.0
 
 =head1 SYNOPSIS
 
@@ -154,10 +158,10 @@ version v0.999.998.5
     bulk->find( { a => 1 } )->update_one( { '$inc' => { x => 1 } } );
 
     # Update all documents matching the selector
-    bulk->find( { a => 2 } )->update( { '$inc' => { x => 2 } } );
+    bulk->find( { a => 2 } )->update_many( { '$inc' => { x => 2 } } );
 
     # Update all documents
-    bulk->find( {} )->update( { '$inc' => { x => 2 } } );
+    bulk->find( {} )->update_many( { '$inc' => { x => 2 } } );
 
     # Replace entire document (update with whole doc replace)
     bulk->find( { a => 3 } )->replace_one( { x => 3 } );
@@ -166,19 +170,19 @@ version v0.999.998.5
     bulk->find( { a => 1 } )->upsert()->update_one( { '$inc' => { x => 1 } } );
 
     # Update all documents matching the selector or upsert
-    bulk->find( { a => 2 } )->upsert()->update( { '$inc' => { x => 2 } } );
+    bulk->find( { a => 2 } )->upsert()->update_many( { '$inc' => { x => 2 } } );
 
     # Replaces a single document matching the selector or upsert
     bulk->find( { a => 3 } )->upsert()->replace_one( { x => 3 } );
 
     # Remove a single document matching the selector
-    bulk->find( { a => 4 } )->remove_one();
+    bulk->find( { a => 4 } )->delete_one();
 
     # Remove all documents matching the selector
-    bulk->find( { a => 5 } )->remove();
+    bulk->find( { a => 5 } )->delete_many();
 
     # Remove all documents
-    bulk->find( {} )->remove();
+    bulk->find( {} )->delete_many();
 
 =head1 DESCRIPTION
 
@@ -193,15 +197,15 @@ exception will be thrown on error.
 
 =head1 METHODS
 
-=head2 remove
+=head2 delete_many
 
-    $bulk->remove;
+    $bulk->delete_many;
 
 Removes all documents matching the query document.
 
-=head2 remove_one
+=head2 delete_one
 
-    $bulk->remove_one;
+    $bulk->delete_one;
 
 Removes a single document matching the query document.
 
@@ -212,9 +216,9 @@ Removes a single document matching the query document.
 Replaces the document matching the query document.  The document
 to replace must not have any keys that begin with a dollar sign, C<$>.
 
-=head2 update
+=head2 update_many
 
-    $bulk->update( $modification );
+    $bulk->update_many( $modification );
 
 Updates all documents  matching the query document.  The modification
 document must have all its keys begin with a dollar sign, C<$>.
@@ -243,11 +247,11 @@ David Golden <david@mongodb.com>
 
 =item *
 
-Mike Friedman <friedo@mongodb.com>
+Mike Friedman <friedo@friedo.com>
 
 =item *
 
-Kristina Chodorow <kristina@mongodb.com>
+Kristina Chodorow <k.chodorow@gmail.com>
 
 =item *
 
