@@ -20,102 +20,90 @@ package MongoDB::Op::_BatchInsert;
 # MongoDB::InsertManyResult
 
 use version;
-our $VERSION = 'v0.999.998.5';
+our $VERSION = 'v1.0.0';
 
-use Moose;
+use Moo;
 
 use MongoDB::BSON;
 use MongoDB::Error;
 use MongoDB::InsertManyResult;
+use MongoDB::OID;
+use MongoDB::_Constants;
 use MongoDB::_Protocol;
-use MongoDB::_Types -types;
-use Types::Standard -types;
-use Safe::Isa;
+use Types::Standard qw(
+    Str
+    ArrayRef
+    Bool
+);
 use Scalar::Util qw/blessed reftype/;
 use Tie::IxHash;
-use namespace::clean -except => 'meta';
+use namespace::clean;
 
 has db_name => (
     is       => 'ro',
-    isa      => Str,
     required => 1,
+    isa      => Str,
 );
 
 has coll_name => (
     is       => 'ro',
-    isa      => Str,
     required => 1,
+    isa      => Str,
 );
 
-# may or may not have _id; caller must get it right. E.g. index creation
-# on legacy mongod does not use _id
+# may or may not have _id; will be added if check_keys is true
 has documents => (
     is       => 'ro',
-    isa      => ArrayRef,
     required => 1,
+    isa      => ArrayRef,
 );
 
 has ordered => (
-    is      => 'ro',
-    isa     => Bool,
-    default => 1,
+    is       => 'ro',
+    required => 1,
+    isa      => Bool,
 );
 
 has check_keys => (
-    is      => 'ro',
-    isa     => Bool,
-    default => 1,
+    is       => 'ro',
+    required => 1,
+    isa      => Bool,
 );
 
-with qw/MongoDB::Role::_WriteOp/;
+# starts empty and gets initialized during operations
+has _doc_ids => (
+    is       => 'ro',
+    writer   => '_set_doc_ids',
+    init_arg => undef,
+    isa      => ArrayRef,
+);
 
-sub BUILD {
-    my ($self) = @_;
-
-    # coerce documents to IxHash or die trying
-    # XXX eventually, do this with a Types::Standard coercion?
-    for my $d ( @{ $self->documents } ) {
-        if ( $d->$_isa('Tie::IxHash') ) {
-            next;
-        }
-        elsif ( ref($d) eq 'ARRAY' ) {
-            $d = Tie::IxHash->new(@$d);
-        }
-        elsif ( ref($d) eq 'HASH' ) {
-            $d = Tie::IxHash->new(%$d);
-        }
-        elsif ( blessed($d) && reftype($d) eq 'HASH' ) {
-            $d = Tie::IxHash->new(%$d);
-        }
-        else {
-            MongoDB::DocumentError->throw(
-                message  => "Can't insert document of type " . ref($d),
-                document => $d,
-            );
-        }
-    }
-
-    return;
-}
+with $_ for qw(
+  MongoDB::Role::_PrivateConstructor
+  MongoDB::Role::_WriteOp
+  MongoDB::Role::_InsertPreEncoder
+);
 
 sub execute {
     my ( $self, $link ) = @_;
 
-    # XXX until we have a proper BSON::Raw class, we bless on the fly
-    my $max_size = $link->max_bson_object_size;
+    my $documents = $self->documents;
+    my $invalid_chars = $self->check_keys ? '.' : '';
 
-    my $ck = $self->check_keys;
-    my $insert_docs = [
-        map {
-            my $s = MongoDB::BSON::encode_bson( $_, $ck, $max_size );
-            bless \$s, "MongoDB::BSON::Raw";
-        } @{ $self->documents }
-    ];
+    my (@insert_docs, @ids);
+
+    my $last_idx = $#$documents;
+    for ( my $i = 0; $i <= $last_idx; $i++ ) {
+        push @insert_docs, $self->_pre_encode_insert( $link, $documents->[$i], $invalid_chars );
+        push @ids, $insert_docs[-1]{metadata}{_id};
+    }
+
+    $self->_set_doc_ids(\@ids);
 
     my $res =
-        $link->accepts_wire_version(2)
-      ? $self->_command_insert( $link, $insert_docs )
-      : $self->_legacy_op_insert( $link, $insert_docs );
+        $link->does_write_commands
+      ? $self->_command_insert( $link, \@insert_docs )
+      : $self->_legacy_op_insert( $link, \@insert_docs );
 
     $res->assert;
     return $res;
@@ -142,7 +130,7 @@ sub _legacy_op_insert {
 
     my $ns = $self->db_name . "." . $self->coll_name;
     my $op_bson =
-      MongoDB::_Protocol::write_insert( $ns, join( "", map { $$_ } @$insert_docs ) );
+      MongoDB::_Protocol::write_insert( $ns, join( "", map { $_->{bson} } @$insert_docs ) );
 
     return $self->_send_legacy_op_with_gle( $link, $op_bson, undef,
         "MongoDB::InsertManyResult" );
@@ -151,11 +139,9 @@ sub _legacy_op_insert {
 sub _parse_cmd {
     my ( $self, $res ) = @_;
     return unless $res->{ok};
-    my $ids = {};
-    for my $i ( 0 .. $#{ $self->documents } ) {
-        $ids->{$i} = $self->documents->[$i]->FETCH("_id");
-    }
-    return ( inserted_ids => $ids );
+    my $inserted = $self->_doc_ids;
+    my $ids = [ map +{ index => $_,  _id => $inserted->[$_] }, 0 .. $#{$inserted} ];
+    return ( inserted_count => scalar @$inserted, inserted => $ids );
 }
 
 BEGIN {
