@@ -20,46 +20,47 @@ package MongoDB::Op::_GetMore;
 # (after inflation from BSON)
 
 use version;
-our $VERSION = 'v0.999.998.5';
+our $VERSION = 'v1.0.0';
 
-use Moose;
+use Moo;
 
-use MongoDB::_Types -types;
-use Types::Standard -types;
+use MongoDB::_Constants;
+use Types::Standard qw(
+    InstanceOf
+    Num
+    Str
+);
 use MongoDB::_Protocol;
-use namespace::clean -except => 'meta';
+use namespace::clean;
 
 has ns => (
     is       => 'ro',
-    isa      => Str,
     required => 1,
+    isa      => Str,
 );
 
 has client => (
     is       => 'ro',
-    isa      => InstanceOf['MongoDB::MongoClient'],
-    required => 1,
-);
-
-has bson_codec => (
-    is       => 'ro',
-    isa      => InstanceOf['MongoDB::MongoClient'], # XXX only for now
     required => 1,
+    isa      => InstanceOf ['MongoDB::MongoClient'],
 );
 
 has cursor_id => (
     is       => 'ro',
-    isa      => Str,
     required => 1,
+    isa      => Str,
 );
 
 has batch_size => (
-    is      => 'ro',
-    isa     => Num,
-    default => 0,
+    is       => 'ro',
+    required => 1,
+    isa      => Num,
 );
 
-with 'MongoDB::Role::_DatabaseOp';
+with $_ for qw(
+  MongoDB::Role::_PrivateConstructor
+  MongoDB::Role::_DatabaseOp
+);
 
 sub execute {
     my ( $self, $link ) = @_;
