@@ -19,11 +19,11 @@ package MongoDB::Role::_Cursor;
 # Role providing a cursor interface
 
 use version;
-our $VERSION = 'v0.999.998.5';
+our $VERSION = 'v1.0.0';
 
-use Moose::Role;
-use namespace::clean -except => 'meta';
+use Moo::Role;
+use namespace::clean;
 
-requires qw/all has_next info next/;
+requires qw/all has_next next/;
 
 1;
