@@ -19,24 +19,25 @@ package MongoDB::Role::_ReadOp;
 # MongoDB role for read ops that provides read preference
 
 use version;
-our $VERSION = 'v0.999.998.5';
+our $VERSION = 'v1.0.0';
 
-use Moose::Role;
+use Moo::Role;
 
 use MongoDB::ReadPreference;
-use MongoDB::_Types -types;
-use Types::Standard -types;
-use namespace::clean -except => 'meta';
-
-my $PRIMARY = MongoDB::ReadPreference->new;
+use MongoDB::_Constants;
+use MongoDB::_Types qw(
+    ReadPreference
+);
+use Types::Standard qw(
+    Maybe
+);
+use namespace::clean;
 
 with 'MongoDB::Role::_DatabaseOp';
 
 has read_preference => (
-    is      => 'ro',
-    isa     => ReadPreference,
-    coerce  => 1,
-    default => sub { $PRIMARY },
+    is  => 'ro',
+    isa => Maybe [ReadPreference],
 );
 
 1;
