@@ -19,12 +19,19 @@ package MongoDB::WriteConcern;
 # ABSTRACT: Encapsulate and validate a write concern
 
 use version;
-our $VERSION = 'v0.999.998.5';
+our $VERSION = 'v1.0.0';
 
-use Moose;
+use Moo;
 use MongoDB::Error;
-use MongoDB::_Types -types;
-use Types::Standard -types;
+use MongoDB::_Types qw(
+    Booleanpm
+);
+use Types::Standard qw(
+    Bool
+    HashRef
+    Num
+    Str
+);
 use Scalar::Util qw/looks_like_number/;
 use namespace::clean -except => 'meta';
 
@@ -68,54 +75,53 @@ has wtimeout => (
 has j => (
     is        => 'ro',
     isa       => Booleanpm,
-    coerce    => 1,
+    coerce    => Booleanpm->coercion,
     predicate => '_has_j',
 );
 
-has _is_safe => (
-    is      => 'ro',
+has _is_acknowledged => (
+    is      => 'lazy',
     isa     => Bool,
-    lazy    => 1,
-    reader  => 'is_safe',
-    builder => '_build_is_safe',
+    reader  => 'is_acknowledged',
+    builder => '_build_is_acknowledged',
 );
 
 has _as_struct => (
-    is      => 'ro',
+    is      => 'lazy',
     isa     => HashRef,
-    lazy    => 1,
     reader  => 'as_struct',
     builder => '_build_as_struct',
 );
 
-sub _build_is_safe {
+sub _build_is_acknowledged {
     my ($self) = @_;
-    return !!( $self->j || $self->_w_is_safe );
+    return !!( $self->j || $self->_w_is_acknowledged );
 }
 
 sub _build_as_struct {
     my ($self) = @_;
     return {
-        ( $self->_has_w        ? ( w        => $self->w )        : () ),
-        ( $self->_has_wtimeout ? ( wtimeout => $self->wtimeout ) : () ),
-        ( $self->_has_j        ? ( j        => $self->j )        : () ),
+        ( $self->_has_w        ? ( w        => $self->w )           : () ),
+        ( $self->_has_wtimeout ? ( wtimeout => 0+ $self->wtimeout ) : () ),
+        ( $self->_has_j        ? ( j        => $self->j )           : () ),
     };
 }
 
 sub BUILD {
     my ($self) = @_;
-    if ( ! $self->_w_is_safe && $self->j ) {
+    if ( ! $self->_w_is_acknowledged && $self->j ) {
         MongoDB::UsageError->throw("can't use write concern w=0 with j=" . $self->j );
     }
     return;
 }
 
-sub _w_is_safe {
+sub _w_is_acknowledged {
     my ($self) = @_;
     return $self->_has_w
       && ( looks_like_number( $self->w ) ? $self->w > 0 : length $self->w );
 }
 
+
 1;
 
 __END__
@@ -130,7 +136,7 @@ MongoDB::WriteConcern - Encapsulate and validate a write concern
 
 =head1 VERSION
 
-version v0.999.998.5
+version v1.0.0
 
 =head1 SYNOPSIS
 
@@ -178,11 +184,11 @@ David Golden <david@mongodb.com>
 
 =item *
 
-Mike Friedman <friedo@mongodb.com>
+Mike Friedman <friedo@friedo.com>
 
 =item *
 
-Kristina Chodorow <kristina@mongodb.com>
+Kristina Chodorow <k.chodorow@gmail.com>
 
 =item *
 
