@@ -17,14 +17,24 @@
 package MongoDB::_Server;
 
 use version;
-our $VERSION = 'v0.999.998.5';
+our $VERSION = 'v1.0.0';
 
-use Moose;
-use MongoDB::_Types -types;
-use Types::Standard -types;
+use Moo;
+use MongoDB::_Types qw(
+    NonNegNum
+    HostAddress
+    ServerType
+    HostAddressList
+);
+use Types::Standard qw(
+    Bool
+    InstanceOf
+    HashRef
+    Str
+    Num
+);
 use List::Util qw/first/;
-use Syntax::Keyword::Junction qw/any none/;
-use Time::HiRes qw/tv_interval/;
+use Time::HiRes qw/time/;
 use namespace::clean -except => 'meta';
 
 # address: the hostname or IP, and the port number, that the client connects
@@ -34,7 +44,7 @@ use namespace::clean -except => 'meta';
 has address => (
     is       => 'ro',
     isa      => HostAddress,
-    coerce   => 1,
+    coerce   => HostAddress->coercion,
     required => 1,
 );
 
@@ -42,7 +52,7 @@ has address => (
 
 has last_update_time => (
     is       => 'ro',
-    isa      => ArrayRef, # [ Time::HighRes::gettimeofday() ]
+    isa      => Num, # floating point time
     required => 1,
 );
 
@@ -56,7 +66,7 @@ has error => (
 
 # roundTripTime: the duration of the ismaster call. Default null.
 
-has rtt_ms => (
+has rtt_sec => (
     is      => 'ro',
     isa     => NonNegNum,
     default => 0,
@@ -84,9 +94,8 @@ has is_master => (
 # - PossiblePrimary	Not yet checked, but another member thinks it is the primary.
 
 has type => (
-    is      => 'ro',
+    is      => 'lazy',
     isa     => ServerType,
-    lazy    => 1,
     builder => '_build_type',
     writer  => '_set_type',
 );
@@ -122,23 +131,35 @@ sub _build_type {
 
 for my $s (qw/hosts passives arbiters/) {
     has $s => (
-        is      => 'ro',
+        is      => 'lazy',
         isa     => HostAddressList,
-        lazy    => 1,
         builder => "_build_$s",
-        coerce  => 1,
+        coerce  => HostAddressList->coercion,
     );
 
     no strict 'refs';
     *{"_build_$s"} = sub { $_[0]->is_master->{$s} || [] };
 }
 
+
+# address configured as part of replica set: string or null. Default null.
+
+has me => (
+    is      => 'lazy',
+    isa     => Str,
+    builder => "_build_me",
+);
+
+sub _build_me {
+    my ($self) = @_;
+    return $self->is_master->{me} || '';
+}
+
 # setName: string or null. Default null.
 
 has set_name => (
-    is      => 'ro',
+    is      => 'lazy',
     isa     => Str,
-    lazy    => 1,
     builder => "_build_set_name",
 );
 
@@ -151,9 +172,8 @@ sub _build_set_name {
 # null.
 
 has primary => (
-    is      => 'ro',
+    is      => 'lazy',
     isa     => Str,           # not HostAddress -- might be empty string
-    lazy    => 1,
     builder => "_build_primary",
 );
 
@@ -165,9 +185,8 @@ sub _build_primary {
 # tags: (a tag set) map from string to string. Default empty.
 
 has tags => (
-    is      => 'ro',
+    is      => 'lazy',
     isa     => HashRef,
-    lazy    => 1,
     builder => "_build_tags",
 );
 
@@ -177,21 +196,33 @@ sub _build_tags {
 }
 
 has is_available => (
-    is      => 'ro',
+    is      => 'lazy',
     isa     => Bool,
-    lazy    => 1,
     builder => "_build_is_available",
 );
 
 sub _build_is_available {
     my ($self) = @_;
-    return $self->type eq none(qw/Unknown PossiblePrimary/);
+    return $self->type ne 'Unknown' && $self->type ne 'PossiblePrimary';
+}
+
+has is_readable => (
+    is      => 'lazy',
+    isa     => Bool,
+    builder => "_build_is_readable",
+);
+
+# any of these can take reads. Topologies will screen inappropriate
+# ones out. E.g. "Standalone" won't be found in a replica set topology.
+sub _build_is_readable {
+    my ($self) = @_;
+    my $type = $self->type;
+    return !! grep { $type eq $_ } qw/Standalone RSPrimary RSSecondary Mongos/;
 }
 
 has is_writable => (
-    is      => 'ro',
+    is      => 'lazy',
     isa     => Bool,
-    lazy    => 1,
     builder => "_build_is_writable",
 );
 
@@ -199,12 +230,13 @@ has is_writable => (
 # ones out. E.g. "Standalone" won't be found in a replica set topology.
 sub _build_is_writable {
     my ($self) = @_;
-    return $self->type eq any(qw/Standalone RSPrimary Mongos/);
+    my $type = $self->type;
+    return !! grep { $type eq $_ } qw/Standalone RSPrimary Mongos/;
 }
 
 sub updated_since {
-    my ( $self, $tv ) = @_;
-    return tv_interval( $tv, $self->last_update_time ) > 0;
+    my ( $self, $time ) = @_;
+    return( ($self->last_update_time - $time) > 0 );
 }
 
 # check if server matches a single tag set (NOT a tag set list)
@@ -225,16 +257,28 @@ sub matches_tag_set {
 
 sub status_string {
     my ($self) = @_;
-    if ( $self->error ) {
+    if ( my $err = $self->error ) {
+        $err =~ tr[\n][ ];
         return
-          sprintf( "%s (type: %s, error: %s)", map { $self->$_ } qw/address type error/ );
+          sprintf( "%s (type: %s, error: %s)", $self->{address}, $self->{type}, $err);
     }
     else {
         return sprintf( "%s (type: %s)", map { $self->$_ } qw/address type/ );
     }
 }
 
-__PACKAGE__->meta->make_immutable;
+sub status_struct {
+    my ($self) = @_;
+    my $info = {
+        address          => $self->address,
+        type             => $self->type,
+        last_update_time => $self->last_update_time,
+    };
+    $info->{error} = $self->error         if $self->error;
+    $info->{tags}  = { %{ $self->tags } } if %{ $self->tags };
+    return $info;
+}
+
 
 1;
 
